/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *     																	   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "UCQFontBar.h"
/**
 * \brief Loads the icons for the bold and italic actions and populates the fontsize
 *        combo with int values between 6 and 96.
 *
 * \param parent Pointer to the parent QWidget.
 */
UCQFontBar::UCQFontBar ( QWidget *parent ) :
    QToolBar ( parent )
{
    QIcon boldActionIcon ( ":icons/format-text-bold.svg" );
    QIcon italicActionIcon ( ":icons/format-text-italic.svg" );
    fontComboBox_ = new QFontComboBox ( this );
    sizeComboBox_ = new QComboBox ( this );

    connect ( fontComboBox_, SIGNAL ( currentFontChanged ( const QFont& ) ),
              this,          SIGNAL ( currentFontChanged ( const QFont& ) ));

    connect ( fontComboBox_, SIGNAL ( currentFontChanged ( const QFont& ) ),
              this,          SLOT   ( saveCurrentFont    ( const QFont& ) ));

    connect ( sizeComboBox_, SIGNAL ( currentIndexChanged ( const QString& ) ),
              this,          SIGNAL ( currentSizeChanged  ( const QString& )));


    // populate the combobox with size-values
    QStringList sizes;
    for ( int i = 6; i < 17; i++ )
    {
        sizes << QString::number ( i, 10 );
    }
    for ( int i = 18; i <= 28; i = i + 2 )
    {
        sizes << QString::number ( i, 10 );
    }
    for ( int i = 32; i <= 48; i = i + 4 )
    {
        sizes << QString::number ( i, 10 );
    }
    for ( int i = 54; i <= 66; i = i + 6 )
    {
        sizes << QString::number ( i, 10 );
    }
    for ( int i = 72; i <= 96; i = i + 8 )
    {
        sizes << QString::number ( i, 10 );
    }

    addWidget ( fontComboBox_ );
    addWidget ( sizeComboBox_ );

    sizeComboBox_->addItems ( sizes );
    sizeComboBox_->setEditable ( true );
    // does not work with some Qt-styles
    sizeComboBox_->setMaxVisibleItems ( 6 );
    sizeComboBox_->setCurrentIndex ( 13 );

    fontComboBox_->setEditable ( true );
    // does not work with some Qt-styles
    fontComboBox_->setMaxVisibleItems ( 10 );

    actionItalic_ = addAction ( italicActionIcon, "" );
    actionItalic_->setToolTip ( tr ( "Italic" ) );

    actionBold_ = addAction ( boldActionIcon, "" );
    actionBold_->setToolTip ( tr ( "Bold" ) );
    actionItalic_->setCheckable ( true );
    actionBold_->setCheckable ( true );

    connect ( actionBold_, SIGNAL ( toggled      ( bool ) ),
              this,        SIGNAL ( bold_toggled ( bool ) ) );

    connect ( actionItalic_, SIGNAL( toggled        ( bool ) ),
              this,          SIGNAL( italic_toggled ( bool )));
}

/**
 * \brief Slot to en-/disable the four contained inputs.
 *
 * \param enable If true the controlls of the bar are enabled.
 *
 * \see MainWindow::updateStates()
 */
void UCQFontBar::enableActions ( bool enable )
{
    fontComboBox_->setEnabled ( enable );
    sizeComboBox_->setEnabled ( enable );
    actionBold_->setEnabled   ( enable );
    actionItalic_->setEnabled ( enable );
}

/**
 * \brief Store the chosen font in the QSettings.
 *
 * \param font The font to store.
 */
void UCQFontBar::saveCurrentFont ( const QFont& font )
{
    QSettings settings ( "Unicode Viewer", "Unicode Viewer" );
    settings.setValue  ( "font", font.family () );
}

/**
 * \brief Initializes the font.
 *
 * Hierarchy is set to Code2000, DejaVu Sans, Arial. If the user in a previous
 * session, that font will be selected.
 *
 */
void UCQFontBar::initFont ()
{
    QStringList fonts;
    QSettings settings ( "Unicode Viewer", "Unicode Viewer" );
    fonts << settings.value ( "font" ).toString ();
    fonts << "Code2000";
    fonts << "DejaVu Sans";
    fonts << "Arial";
    bool fontMatched = false;
    for ( int i = 0; i < fonts.size (); ++i )
    {
        QFont f ( fonts.at ( i ) );
        if ( f.exactMatch () )
        {
            fontComboBox_->setCurrentFont ( f );
            fontMatched = true;
            break;
        }
    }

    if ( !fontMatched )
    {
        fontComboBox_->setCurrentFont ( QApplication::font () );
    }
}

/**
 * \brief Selects the given font in the font combo.
 *
 * \param font The font to select.
 */
void UCQFontBar::selectFont( const QFont& font){
    fontComboBox_->setCurrentFont ( font );
}

/**
 * \brief Returns the font curretly selected in the font combo.
 *
 * \return The font currently selected in the font combo.
 */
QFont UCQFontBar::getFont(){
    return fontComboBox_->currentFont();
}
