/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *     																	   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef UNICODETABLEVIEWER_H
#define UNICODETABLEVIEWER_H

#include <QtGui/QWidget>
#include <QFileSystemWatcher>
#include <QFileDialog>
#include <QDir>
#include <QMessageBox>
#include <QSettings>
#include <QDir>
#include <QActionGroup>
#include <QFontDialog>
#include "ui/ui_mainwindow.h"
#include "io/TableIO.h"
#include "controller/LuaController.h"
#include "controller/TableController.h"
#include "controller/SortController.h"
#include "lua/LuaSearch.h"
#include "lua/LuaOutput.h"
#include "lua/LuaSort.h"
#include "data/TableModel.h"
#include "gui/UCQTableView.h"
#include "gui/UCQSearchDialog.h"
#include "gui/UCQBlockDialog.h"
#include "gui/UCQDetails.h"
#include "gui/UCQWarnings.h"
#include "gui/UCQStatusBar.h"
#include "gui/UCQHelpBrowser.h"
#include "tools/UCQExecuteLuaThread.h"

/**
 * \class MainWindow
 * \brief The main window of the application.
 *
 * \author Martin Burgemeister <Martin.Burgemeister@gmx.de>,
 *         Mathis Lindemann <MathisLindemann@gmx.net>
 * \version 1.0
 * \date 16 August 2010
 *
 * \see UCQConsole
 * \see UCQBlockList
 * \see UCQTableView
 * \see UCQDetailsBar
 * \see UCQStatusBar
 * \see UCQToolBar
 * \see UCQFontBar
 * \see UCQSearchDialog
 * \see UCQTableToolBar
 * \see UCQBlockDialog
 * \see UCQExecuteFileThread
 * \see SearchController
 * \see LuaController
 * \see TableModel
 */
class MainWindow: public QMainWindow
{

Q_OBJECT

public:
    MainWindow  ( QWidget* parent = 0 );
    ~MainWindow ();

protected:
    void closeEvent ( QCloseEvent* );

private:
    void initLua       ();
    void initMenu      ();
    void initStatusBar ();
    void initConsoleFontSelection();

    void setCurrentFile  ( const QString& );
    void setCurrentBlock ( const QString& );

    void updateRecentFileActions   ();
    void updateRecentBlocksActions ();

    void loadTable      ( QString, bool = false );
    void loadBlockTable ( QString, bool = false );

    void writeSettings ();
    void readSettings  ();

    /**
     * \brief The object of the designed main window.
     */
    Ui::MainWindow ui_;
    /**
     * \brief The LuaController.
     */
    LuaController* lua_;
    /**
     * \brief The TableModel.
     */
    TableModel* tableModel_;
    /**
     * \brief The TableController.
     */
    TableController* tableControl_;
    /**
     * \brief  Max Number of recent Unicode files.
     */
    enum
    {
        MaxRecentFiles = 5
    };
    /**
     *  \brief Array of QActions for the recent Unicode files.
     */
    QAction* recentFileActions_[ MaxRecentFiles ];
    /**
     *  \brief Array of QActions for the recent block files.
     */
    QAction* recentBlocksActions_[ MaxRecentFiles ];
    /**
     *  \brief QStringList with the names of the recent Unicode files.
     */
    QStringList recentFiles_;
    /**
     *  \brief QStringList with the names of the recent block files.
     */
    QStringList recentBlocks_;
    /**
     *  \brief Current block file name.
     */
    QString curBlock_;
    /**
     *  \brief Current file name.
     */
    QStringList curFile_;
    /**
     * \brief Current Unicode table directory.
     */
    QString curTableDir_;
    /**
     * \brief Current blocks directory.
     */
    QString curBlocksDir_;
    /**
     * \brief Current output directory.
     */
    QString curOutputDir_;
    /**
     * \brief Current Lua files directory.
     */
    QString curLuaDir_;
    /**
     *  \brief Search controller.
     */
    SearchController* sc_;
    /**
     * \brief Details widget.
     */
    UCQDetails* details_;
    /**
     * \brief The search dialog.
     */
    UCQSearchDialog* ucqsd_;
    /**
     * \brief The status bar.
     */
    UCQStatusBar* statusBar_;
    /**
     * \brief The warnings dialog.
     */
    UCQWarnings* warningsDialog_;
    /**
     * \brief The help browser.
     */
    UCQHelpBrowser* helpBrowser_;
    /**
     * \brief Thread for loading Lua files.
     */
    UCQExecuteLuaThread executeLuaThread_;
    /**
     * \brief When true, the mainWindow will not accept close events.
     */
    bool working_;
    /**
     * \brief Watches the loaded files on disk.
     */
    QFileSystemWatcher fileSystemWatcher;
    /**
     * \brief Example controller for a sorting functionality.
     */
    SortController* sortControl_;
    /**
     * \brief Exclusive action group for the menu actions that change the
     *        sorting in the blocklist.
     */
    QActionGroup *blockSortingActions;
    /**
      * \brief Exclusive action group for the menu actions that change the
      *        sorting order in the table.
      */
    QActionGroup *tableSortingActionsOrders;
    /**
     * \brief Exclusive action group for the menu actions that change the
     *        sorting attribute in the table.
     */
    QActionGroup *tableSortingActionsAttributes;

    /**
     * \brief Arrow up icon for the sort order button.
     */
    QIcon* sortAscIcon_;
    /**
     * \brief Arrow down icon for the sort order button.
     */
    QIcon* sortDescIcon_;
    /**
     * \brief Current sort order.
     *
     * If true, next use of the sort button will sort in ascending order.
     */
    bool ascending_;

protected slots:
    void openRecentFile  ();
    void openRecentBlock ();

    void on_pushButtonLoadLua_clicked ();

    void showBlockDialog ();

    void on_actionReset_triggered           ();
    void on_actionAddUcTable_triggered      ();
    void on_actionUcTableLoad_triggered     ();
    void on_actionUcTableSave_triggered     ();
    void on_actionAddBlocksTable_triggered  ();
    void on_actionBlocksTableLoad_triggered ();

    void on_actionSearch_triggered         ();
    void on_actionSearchNext_triggered     ();
    void on_actionSearchPrevious_triggered ();

    void on_actionHelp_triggered  ();
    void on_actionAbout_triggered ();

    void on_actionBlocksNameAsc_triggered();
    void on_actionBlocksNameDesc_triggered();
    void on_actionBlocksCodeAsc_triggered();
    void on_actionBlocksCodeDesc_triggered();

    void on_sortOrderButton_clicked();
    void on_sortButton_clicked();

    void on_actionLoadFont_triggered ();

    void clearTable          ();
    void cutSelectionAction  ();
    void cropSelectionAction ();

    void updateStates ();
public slots:
    void show ();

private slots:
    void loadOldFiles ();
    void fileChanged  ( QString );
    void reloadFiles  ();

signals:
    /**
     * \brief Signal to stop the execution from the script.
     */
    void stopExecution ();
    /**
     * \brief Signal for sorting the blocks in the UCQBlockList.
     */
    void sortBlocks( UCQBlockList::SortingOrders );
};

#endif // UNICODETABLEVIEWER_H
